@@ -10,7 +10,6 @@
 use strict;
 use warnings;
 
-use Mail::DKIM::PrivateKey;
 use Mail::DKIM::Canonicalization::dk_simple;
 use Mail::DKIM::Canonicalization::dk_nofws;
 
@@ -18,7 +17,7 @@ package Mail::DKIM::Algorithm::dk_rsa_sha1;
 use base "Mail::DKIM::Algorithm::Base";
 use Carp;
 use MIME::Base64;
-use Digest::SHA1;
+use Digest::SHA;
 
 sub finish_header
 {
@@ -33,10 +32,14 @@ sub finish_header
 		my $author = $self->{canon}->{interesting_header}->{from};
 		$author = defined($author) && (Mail::Address->parse($author))[0];
 
-		$s->init_identity(
-			$sender ? $sender->address :
-			$author ? $author->address :
-			undef);
+		if ($sender)
+		{
+			$s->init_identity($sender->address, "header.sender");
+		}
+		elsif ($author)
+		{
+			$s->init_identity($author->address, "header.from");
+		}
 	}
 	return;
 }
@@ -79,7 +82,7 @@ sub init_digests
 	my $self = shift;
 
 	# initialize a SHA-1 Digest
-	$self->{header_digest} = new Digest::SHA1;
+	$self->{header_digest} = Digest::SHA->new(1);
 	$self->{body_digest} = $self->{header_digest};
 }
 
@@ -90,7 +93,7 @@ sub sign
 	my ($private_key) = @_;
 
 	my $digest = $self->{header_digest}->digest;
-	my $signature = $private_key->sign_sha1_digest($digest);
+	my $signature = $private_key->sign_digest("SHA-1", $digest);
 
 	return encode_base64($signature, "");
 }
@@ -105,7 +108,7 @@ sub verify
 
 	my $digest = $self->{header_digest}->digest;
 	my $sig = decode_base64($base64);
-	return $public_key->verify_sha1_digest($digest, $sig);
+	return $public_key->verify_digest("SHA-1", $digest, $sig);
 }
 
 sub finish_message
@@ -121,4 +124,9 @@ sub finish_message
 	}
 }
 
+sub wants_pre_signature_headers
+{
+	return 0;
+}
+
 1;
