@@ -61,48 +61,56 @@ sub fetch
 	return $self;
 }
 
-#EXPERIMENTAL1
-# my $x = Mail::DKIM::PublicKey->fetch_async(
-#                     Protocol => "dns",
-#                     etc.,
-#                   );
-# my $public_key = $x->();
-
-#EXPERIMENTAL2
-# my ($pubk, $E);
-# my $x = Mail::DKIM::PublicKey->fetch_async(
-#                     Protocol => "dns",
-#                     etc.,
-#                     Callbacks => { },
-#                   );
-# return $x->();
-
+# fetch_async() - asynchronously tries fetching a specific public key
+# using a specific protocol.
+#
+# Usage:
+#   my $fut = Mail::DKIM::PublicKey->fetch_async(
+#                      Protocol => "dns/txt",
+#                      Selector => "selector1",
+#                      Domain => "example.org",
+#                      Callbacks => { ... }, #optional
+#                      );
+#
+#   # some later time
+#   my $pubkey = $fut->();    # blocks until the public key is returned
+#
 sub fetch_async
 {
 	my $class = shift;
 	my %prms = @_;
 
-	my ($query_type, $query_options) = split(/\//, $prms{Protocol}, 2);
-	if (lc($query_type) ne "dns")
-	{
-		die "unknown query type '$query_type'\n";
-	}
+	defined($prms{Protocol}) && $prms{Protocol} =~ m{^dns(/txt)?$}s
+		or die "invalid/missing Protocol\n";
+
+	defined($prms{Selector}) && length($prms{Selector})
+		or die "invalid/missing Selector\n";
+
+	defined($prms{Domain}) && length($prms{Domain})
+		or die "invalid/missing Domain\n";
 
-	my $host = $prms{Selector} . "._domainkey." . $prms{Domain};
 	my %callbacks = %{$prms{Callbacks} || {}};
 	my $on_success = $callbacks{Success} || sub { $_[0] };
 	$callbacks{Success} = sub {
 			my @resp = @_;
 			unless (@resp)
 			{
-				# no response => NXDOMAIN
+				# no requested resource records or NXDOMAIN,
 				return $on_success->();
 			}
 
 			my $strn;
-			foreach my $ans (@resp) {
-				next unless $ans->type eq "TXT";
-				$strn = join "", $ans->char_str_list;
+			foreach my $rr (@resp) {
+				next unless $rr->type eq "TXT";
+
+				# join with no intervening spaces, RFC 6376
+				if (Net::DNS->VERSION >= 0.69) {
+					# must call txtdata() in a list context
+					$strn = join "", $rr->txtdata;
+				} else {
+					# char_str_list method is 'historical'
+					$strn = join "", $rr->char_str_list;
+				}
 				last;
 			}
 
@@ -119,6 +127,7 @@ sub fetch_async
 	#
 	# perform DNS query for public key...
 	#
+	my $host = $prms{Selector} . "._domainkey." . $prms{Domain};
 	my $waiter = Mail::DKIM::DNS::query_async(
 			$host, "TXT",
 			Callbacks => \%callbacks,
